package com.example


import com.example.*
import io.ktor.http.*
import io.ktor.server.application.*
import io.ktor.server.engine.embeddedServer
import io.ktor.server.netty.Netty
import io.ktor.server.plugins.contentnegotiation.*
import io.ktor.serialization.kotlinx.json.*
import io.ktor.server.request.*
import io.ktor.server.response.*
import io.ktor.server.routing.*
import kotlinx.serialization.KSerializer
import kotlinx.serialization.descriptors.PrimitiveKind
import kotlinx.serialization.descriptors.PrimitiveSerialDescriptor
import kotlinx.serialization.descriptors.SerialDescriptor
import kotlinx.serialization.encoding.Decoder
import kotlinx.serialization.encoding.Encoder
import kotlinx.serialization.json.Json
import kotlinx.serialization.modules.SerializersModule
import kotlinx.serialization.modules.contextual
import java.math.BigDecimal

fun main() {
    embeddedServer(Netty, port = 8080, host = "0.0.0.0") {
        configureSerialization()
        configureRouting()
        DatabaseFactory.init()
    }.start(wait = true)
}

object BigDecimalSerializer : KSerializer<BigDecimal> {
    override val descriptor: SerialDescriptor =
        PrimitiveSerialDescriptor("BigDecimal", PrimitiveKind.STRING)

    override fun serialize(encoder: Encoder, value: BigDecimal) {
        encoder.encodeString(value.toPlainString())
    }

    override fun deserialize(decoder: Decoder): BigDecimal {
        return BigDecimal(decoder.decodeString())
    }
}

fun Application.configureSerialization() {
    install(ContentNegotiation) {
        json(Json {
            serializersModule = SerializersModule {
                contextual(BigDecimalSerializer) // Добавляем поддержку BigDecimal
            }
        })
    }
}

fun Application.configureRouting() {
    routing {
        post("/register") {
            val request = call.receive<UserRegisterRequest>()
            if (AuthService.registerUser(request.name, request.email, request.password)) {
                call.respond(mapOf("message" to "Регистрация успешна"))
            } else {
                call.respond(mapOf("error" to "Email уже зарегистрирован"))
            }
        }

        post("/login") {
            val request = call.receive<UserLoginRequest>()
            if (AuthService.loginUser(request.email, request.password)) {
                call.respond(mapOf("message" to "Вход успешен"))
            } else {
                call.respond(mapOf("error" to "Неверный email или пароль"))
            }
        }

        post("/forgot-password") {
            val request = call.receive<UserForgotPasswordRequest>()
            val code = AuthService.generateResetCode(request.email)

            if (code != null) {
                EmailService.sendResetEmail(request.email, code)
                call.respond(mapOf("message" to "Код на восстановление пароля отправлен"))
            } else {
                call.respond(mapOf("error" to "Email не найден"))
            }
        }

        post("/reset-password") {
            val request = call.receive<UserResetPasswordRequest>()
            if (AuthService.resetPassword(request.code, request.newPassword)) {
                call.respond(mapOf("message" to "Пароль успешно изменён"))
            } else {
                call.respond(mapOf("error" to "Неверный код"))
            }
        }

        post("/reset-passwor") {
            val request = call.receive<UserResetPasswordRequest>()
            if (AuthService.resetPassword(request.code, request.newPassword)) {
                call.respond(mapOf("message" to "Пароль успешно изменён"))
            } else {
                call.respond(mapOf("error" to "Неверный код"))
            }
        }

//        get ("/mainWindow"){
//            val sneakers = AuthService.selectSneakers()
//            call.respond(mapOf("sneaker" to sneakers))
//        }

//        get("/mainWindow") {
//            try {
//                // Ваш код для обработки запроса
//                call.respondText("Main Window Content")
//            } catch (e: Exception) {
//                call.respond(HttpStatusCode.InternalServerError, "Error: ${e.message}")
//            }
//        }

        get("/mainWindow") {
            try {
                val sneakers = AuthService.selectSneakers()
                if (sneakers.isNotEmpty()) {
                    call.respond(mapOf("sneakers" to sneakers))
                } else {
                    call.respond(mapOf("error" to "No sneakers found"))
                }
            } catch (e: Exception) {
                log.error("Error fetching sneakers: ${e.message}", e)
                call.respond(HttpStatusCode.InternalServerError, "Error: ${e.message}")
            }
        }
    }
}
